package com.norco.apicustomertest;

import android.os.Bundle;

import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Spinner;

import com.norco.API.NorcoAPI;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

public class AFCControlActivity extends AppCompatActivity {

    private Button button1;
    private Button button2;
    private Button button3;
    private Button button4;

    private Spinner init_spinner_addr, spinner_node;
    private Spinner open_spinner_addr, open_spinner_door_number;
    private Spinner read_spinner_addr, read_spinner_door_number;

    private static final String TAG = "norcoAPI";
    private NorcoAPI mNorcoAPI = APIApplication.getApplication().getNorcoAPI();

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_afc_control);

        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.setHomeButtonEnabled(true);
            actionBar.setDisplayHomeAsUpEnabled(true);
        }

        initView();
        mNorcoAPI.norcoAPIRegister();
        setTitle(R.string.hw_afc_test);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case android.R.id.home:
            this.finish(); // back button
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        mNorcoAPI.norcoAPIUnregister();
        mNorcoAPI.afcctrl_uart485Close();
    }

    private void initView() {
        button1 = (Button)findViewById(R.id.button1);
        button1.setOnClickListener(uart485Init);

        button2 = (Button)findViewById(R.id.button2);
        button2.setOnClickListener(uart485OpenDoor);

        button3 = (Button)findViewById(R.id.button3);
        button3.setOnClickListener(uart485ReadDoor);

        button4 = (Button)findViewById(R.id.button4);
        button4.setOnClickListener(afc_uart485Close);

        spinner_node = (Spinner)findViewById(R.id.spinner_node);
        init_spinner_addr = (Spinner)findViewById(R.id.init_spinner_addr);
        open_spinner_addr = (Spinner)findViewById(R.id.open_spinner_addr);
        open_spinner_door_number = (Spinner)findViewById(R.id.open_spinner_door_number);
        read_spinner_addr = (Spinner)findViewById(R.id.read_spinner_addr);
        read_spinner_door_number = (Spinner)findViewById(R.id.read_spinner_door_number);

        List<String> allDevices = mNorcoAPI.hwctrl_uartAvailableDevices();
        ArrayAdapter<String> uartDevices = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, allDevices);
        uartDevices.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spinner_node.setAdapter(uartDevices);

        List<String> allAddr = new ArrayList<>();
        List<Integer> allDoor = new ArrayList<>();
        for (int i = 0; i < 16; i++) {
            String hexStr = String.format(Locale.CHINESE, "%02x", i);
            allAddr.add(hexStr);
        }
        for (int i = 1; i < 25; i++) {
            allDoor.add(i);
        }

        ArrayAdapter<String> addrAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, allAddr);
        addrAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        init_spinner_addr.setAdapter(addrAdapter);
        init_spinner_addr.setSelection(3);
        open_spinner_addr.setAdapter(addrAdapter);
        open_spinner_addr.setSelection(3);
        read_spinner_addr.setAdapter(addrAdapter);
        read_spinner_addr.setSelection(3);

        ArrayAdapter<Integer> doorAdapter = new ArrayAdapter<Integer>(this, android.R.layout.simple_spinner_item, allDoor);
        doorAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        open_spinner_door_number.setAdapter(doorAdapter);
        open_spinner_door_number.setSelection(22);
        read_spinner_door_number.setAdapter(doorAdapter);
        read_spinner_door_number.setSelection(22);
    }

    private View.OnClickListener uart485Init = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            String addrHexStr = init_spinner_addr.getSelectedItem().toString();
            int addrInt = Integer.parseInt(addrHexStr, 16);

            int[] addrArr = new int[1]; //数组大小为锁控板个数
            addrArr[0] = addrInt;
            //addrArr[1] = 0x4; //初始化第二块锁控板

            String node = spinner_node.getSelectedItem().toString();
            Log.i(TAG, "afcctrl_uart485Init node=" + node);
            boolean result = mNorcoAPI.afcctrl_uart485Init(node, addrArr);
            Log.i(TAG, "afcctrl_uart485Init result=" + result);
        }
    };

    private View.OnClickListener uart485OpenDoor = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            String addrHexStr = open_spinner_addr.getSelectedItem().toString();
            Log.i(TAG, "afcctrl_uart485OpenDoor addrHexStr=" + addrHexStr);
            int addrInt = Integer.parseInt(addrHexStr, 16);
            Log.i(TAG, "afcctrl_uart485OpenDoor addrInt=" + addrInt);

            int doorNumber = (Integer)open_spinner_door_number.getSelectedItem();
            Log.i(TAG, "afcctrl_uart485OpenDoor doorNumber=" + doorNumber);

            boolean result = mNorcoAPI.afcctrl_uart485OpenDoor(addrInt, doorNumber);
            Log.i(TAG, "afcctrl_uart485OpenDoor result=" + result);
        }
    };

    private View.OnClickListener uart485ReadDoor = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            String addrHexStr = read_spinner_addr.getSelectedItem().toString();
            Log.i(TAG, "uart485ReadDoor addrHexStr=" + addrHexStr);
            int addrInt = Integer.parseInt(addrHexStr, 16);
            Log.i(TAG, "uart485ReadDoor addrInt=" + addrInt);

            int doorNumber = (Integer)read_spinner_door_number.getSelectedItem();
            Log.i(TAG, "uart485ReadDoor doorNumber=" + doorNumber);

            int result = mNorcoAPI.afcctrl_uart485ReadDoor(addrInt, doorNumber);
            Log.i(TAG, "uart485ReadDoor result=" + result);
        }
    };

    private View.OnClickListener afc_uart485Close = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            Log.i(TAG, "afc_uart485Close...");
            mNorcoAPI.afcctrl_uart485Close();
        }
    };
}