package com.norco.apicustomertest;

import android.os.AsyncTask;

import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

/**
 * Author: ZouHaiping on 2019/3/18
 * Desc: 对AsyncTask进行的封装
 */
@SuppressWarnings({"unchecked"})
public class AsyncTaskWrapper<Params, Progress, Result> extends AsyncTask<Params, Progress, Result> {

    private Object mTaskTag;
    private OnLoadListener mListener;
    public static final Executor THREAD_POOL_CACHED = Executors.newCachedThreadPool();
    public static final Executor THREAD_POOL_EXECUTOR = AsyncTask.THREAD_POOL_EXECUTOR;

    public void setTaskTag(Object taskTag) {
        mTaskTag = taskTag;
    }

    public Object getTaskTag() {
        return mTaskTag;
    }

    public void setOnTaskListener(OnLoadListener listener) {
        mListener = listener;
    }

    public void cancle() {
        if (!isCancelled()) {
            cancel(false);
        }
    }

    public boolean hasCancelled() {
        return isCancelled();
    }

    public void updateProgress(Progress... values) {
        publishProgress(values);
    }

    @Override
    protected void onPreExecute() {
        super.onPreExecute();
        mListener.onStart(mTaskTag);
    }

    @Override
    protected Result doInBackground(Params... params) {
        return (Result)mListener.onWorkerThread(mTaskTag, params);
    }

    @Override
    protected void onProgressUpdate(Progress... values) {
        super.onProgressUpdate(values);
        mListener.onProgress(mTaskTag, values);
    }

    @Override
    protected void onPostExecute(Result result) {
        super.onPostExecute(result);
        mListener.onResult(mTaskTag, result);
    }

    @Override
    protected void onCancelled() {
        super.onCancelled();
        mListener.onCancel(mTaskTag);
    }

    public static abstract class OnLoadListener<Params, Progress, Void> {
        public void onStart(Object taskTag) {

        }

        public void onCancel(Object taskTag) {

        }

        public void onProgress(Object taskTag, Progress... values) {

        }

        public void onResult(Object taskTag, Void result) {

        }

        public abstract Void onWorkerThread(Object taskTag, Params... params);
    }
}
