package com.norco.apicustomertest;

import android.os.Bundle;

import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;
import android.widget.ToggleButton;

import com.norco.API.NorcoAPI;
import com.norco.API.util.ByteUtil;
import com.norco.API.util.CanResponseBean;

import java.util.ArrayList;
import java.util.List;

public class CANActivity extends AppCompatActivity {

    private Button sendButton;
    private EditText editText;
    private ToggleButton toggleButton;
    private EditText editTextReceiveData;

    private String canNode;
    private Spinner canSpinner;
    private CanReceiveThread mCanReadThread;
    private static final String TAG = "norcoAPI";
    private NorcoAPI mNorcoAPI = APIApplication.getApplication().getNorcoAPI();

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_hw_can);

        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.setHomeButtonEnabled(true);
            actionBar.setDisplayHomeAsUpEnabled(true);
        }
        setTitle(R.string.hw_can_test);

        initView();
        displaySendData();
    }

    // 界面发送的数据
    private void displaySendData() {
        // 8个字节，16进制字符
        editText.setText("FF112233445566FF");
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case android.R.id.home:
            this.finish();
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onDestroy() {
        if (mCanReadThread != null) {
            Log.i(TAG, "mCanReadThread setStop...");
            mCanReadThread.setStop(true);
            mCanReadThread = null;
        }
        if (canNode != null) {
            mNorcoAPI.hwctrl_canClose(canNode);
        }
        super.onDestroy();
    }

    private void initView() {
        editTextReceiveData = (EditText)findViewById(R.id.editTextRecDisp);
        editText = (EditText)findViewById(R.id.editText);

        sendButton = (Button)findViewById(R.id.sendButton);
        toggleButton = (ToggleButton)findViewById(R.id.toggleButton);

        canSpinner = (Spinner)findViewById(R.id.canSpinner);
        sendButton.setOnClickListener(new ButtonClickEvent());
        toggleButton.setOnCheckedChangeListener(new ToggleButtonCheckedChangeEvent());

        List<String> canList = new ArrayList<>();
        canList.add("can0");
        canList.add("can1");

        ArrayAdapter<String> can0Adapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, canList);
        can0Adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        canSpinner.setAdapter(can0Adapter);
        canSpinner.setSelection(0);
    }

    // 发送按钮
    class ButtonClickEvent implements View.OnClickListener {
        public void onClick(View view) {
            if (view == sendButton) {
                String data = editText.getText().toString();
                byte[] sendByte = ByteUtil.hexToByteArr(data);
                Log.i(TAG, "sendButton byte len=" + sendByte.length);
                if (sendByte.length > 8) {
                    showMessage("CAN Frame发送数据不能超过8字节");
                    return;
                }

                long canID = 0x03;
                mNorcoAPI.hwctrl_canSend(canID, sendByte);
            }
        }
    }

    private class ToggleButtonCheckedChangeEvent implements ToggleButton.OnCheckedChangeListener {
        public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
            int baudrate = 125000;
            canNode = canSpinner.getSelectedItem().toString();

            if (buttonView == toggleButton) {
                if (isChecked) {
                    if (!mNorcoAPI.hwctrl_canOpen(canNode, baudrate)) {
                        showMessage("打开失败，请重试");
                        toggleButton.setChecked(false);
                    } else {
                        showMessage("打开成功");
                        if (mCanReadThread == null) {
                            Log.i(TAG, "mCanReadThread setStart...");
                            mCanReadThread = new CanReceiveThread();
                            mCanReadThread.start();
                        }
                    }
                } else {
                    mNorcoAPI.hwctrl_canClose(canNode);
                }
            }
        }
    }

    // 显示接收数据
    private void displayReceiveData(CanResponseBean data) {
        StringBuilder sMsg = new StringBuilder();
        sMsg.append(" [ID] ");
        sMsg.append(data.getCanid());
        sMsg.append(" [Hex] ");
        sMsg.append(ByteUtil.byteArrToHex(data.getData()));
        sMsg.append("\n");
        editTextReceiveData.append(sMsg);
    }

    // 显示Toast消息
    private void showMessage(String sMsg) {
        Toast.makeText(this, sMsg, Toast.LENGTH_SHORT).show();
    }

    private class CanReceiveThread extends Thread {

        private boolean mStop;

        public void setStop(boolean stop) {
            mStop = stop;
        }

        @Override
        public void run() {
            super.run();
            while (!mStop) {
                final CanResponseBean canFrame = mNorcoAPI.hwctrl_canReceive();
                if (canFrame == null) {
                    try {
                        Thread.sleep(200);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                    continue;
                }

                runOnUiThread(new Runnable() {
                    public void run() {
                        displayReceiveData(canFrame);
                    }
                });
            }
        }
    }
}