package com.norco.apicustomertest;

import android.content.res.AssetManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;

import com.norco.API.NorcoAPI;
import com.norco.API.NorcoNative;
import com.norco.API.util.Util;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;

public class GPIOActivity extends AppCompatActivity {

    private Button buttonExport;
    private Button buttonUnExport;
    private Button buttonDirection;
    private Button buttonSetValue;
    private Button buttonGetValue;
    private Button buttonSetEdge;
    private Button buttonGetEdge;
    private Button buttonStartInterrupt;
    private Button buttonStopInterrupt;

    private EditText editGpioNumber;
    private EditText editExport;
    private EditText editUnExport;
    private EditText editDirection;
    private EditText editSetValue;
    private EditText editGetValue;
    private EditText editGetEdge;

    private Spinner spinnerEdge;

    private static final String TAG = "norcoAPI";
    private NorcoAPI mNorcoAPI = APIApplication.getApplication().getNorcoAPI();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_gpio_control);

        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.setHomeButtonEnabled(true);
            actionBar.setDisplayHomeAsUpEnabled(true);
        }
        setTitle(R.string.hw_gpio_test);

        initView();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case android.R.id.home:
            this.finish();
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    private void initView() {
        editGpioNumber = (EditText)findViewById(R.id.edit_gpio_number);
        buttonExport = (Button)findViewById(R.id.button_export);
        buttonExport.setOnClickListener(mGpioExport);
        editExport = (EditText)findViewById(R.id.edit_export);

        buttonDirection = (Button)findViewById(R.id.button_direction);
        buttonDirection.setOnClickListener(mGpioDirection);
        editDirection = (EditText)findViewById(R.id.edit_direction);

        buttonSetValue = (Button)findViewById(R.id.button_set_value);
        buttonSetValue.setOnClickListener(mGpioSetValue);
        editSetValue = (EditText)findViewById(R.id.edit_set_value);

        buttonGetValue = (Button)findViewById(R.id.button_get_value);
        buttonGetValue.setOnClickListener(mGpioGetValue);
        editGetValue = (EditText)findViewById(R.id.edit_get_value);

        buttonUnExport = (Button)findViewById(R.id.button_unexport);
        buttonUnExport.setOnClickListener(mGpioUnExport);
        editUnExport = (EditText)findViewById(R.id.edit_unexport);

        spinnerEdge = (Spinner)findViewById(R.id.spinner_set_edge);
        editGetEdge = (EditText)findViewById(R.id.edit_get_edge);
        buttonSetEdge = (Button)findViewById(R.id.button_set_edge);
        buttonSetEdge.setOnClickListener(mGpioSetEdge);
        buttonGetEdge = (Button)findViewById(R.id.button_get_edge);
        buttonGetEdge.setOnClickListener(mGpioGetEdge);
        buttonStartInterrupt = (Button)findViewById(R.id.button_start_interrupt);
        buttonStartInterrupt.setOnClickListener(mGpioStartInterrupt);
        buttonStopInterrupt = (Button)findViewById(R.id.button_stop_interrupt);
        buttonStopInterrupt.setOnClickListener(mGpioStopInterrupt);

        List<String> edge = new ArrayList<>();
        edge.add("none");
        edge.add("rising");
        edge.add("falling");
        edge.add("both");
        ArrayAdapter<String> edgeAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, edge);
        edgeAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spinnerEdge.setAdapter(edgeAdapter);
        spinnerEdge.setSelection(1);
    }

    private View.OnClickListener mGpioExport = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            String gpioNumberStr = editGpioNumber.getText().toString();
            if (TextUtils.isEmpty(gpioNumberStr)) {
                showMessage("请输入GPIO索引号");
                return;
            }
            if (!TextUtils.isDigitsOnly(gpioNumberStr)) {
                showMessage("请输入整数GPIO索引号");
                return;
            }


            int gpioNumber = Integer.parseInt(gpioNumberStr);
            Log.i(TAG, "in mGpioExport gpioNumber=" + gpioNumber);
            boolean isSuccess = mNorcoAPI.hwctrl_gpioExport(gpioNumber);
            Log.i(TAG, "in mGpioExport isSuccess=" + isSuccess);
            editExport.setText(isSuccess + "");
        }
    };

    private View.OnClickListener mGpioUnExport = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            String gpioNumberStr = editGpioNumber.getText().toString();
            if (TextUtils.isEmpty(gpioNumberStr)) {
                showMessage("请输入GPIO索引号");
                return;
            }
            if (!TextUtils.isDigitsOnly(gpioNumberStr)) {
                showMessage("请输入整数GPIO索引号");
                return;
            }
            int gpioNumber = Integer.parseInt(gpioNumberStr);


            Log.i(TAG, "in mGpioUnExport gpioNumber=" + gpioNumber);
            boolean isSuccess = mNorcoAPI.hwctrl_gpioUnExport(gpioNumber);
            Log.i(TAG, "in mGpioUnExport isSuccess=" + isSuccess);
            editUnExport.setText(isSuccess + "");
        }
    };

    private View.OnClickListener mGpioDirection = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            String gpioNumberStr = editGpioNumber.getText().toString();
            if (TextUtils.isEmpty(gpioNumberStr)) {
                showMessage("请输入GPIO索引号");
                return;
            }
            if (!TextUtils.isDigitsOnly(gpioNumberStr)) {
                showMessage("请输入整数GPIO索引号");
                return;
            }
            int gpioNumber = Integer.parseInt(gpioNumberStr);


            String directionStr = editDirection.getText().toString();
            if (TextUtils.isEmpty(directionStr)) {
                showMessage("Direction不能为空");
                return;
            }
            if (directionStr.equals("in") || directionStr.equals("out")) {
                Log.i(TAG, "in mGpioDirection directionStr=" + directionStr);
                boolean isSuccess = mNorcoAPI.hwctrl_gpioDirection(gpioNumber, directionStr);
                Log.i(TAG, "in mGpioDirection isSuccess=" + isSuccess);
            } else {
                showMessage("Direction输入in/out");
            }
        }
    };

    private View.OnClickListener mGpioSetValue = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            String gpioNumberStr = editGpioNumber.getText().toString();
            if (TextUtils.isEmpty(gpioNumberStr)) {
                showMessage("请输入GPIO索引号");
                return;
            }
            if (!TextUtils.isDigitsOnly(gpioNumberStr)) {
                showMessage("请输入整数GPIO索引号");
                return;
            }
            int gpioNumber = Integer.parseInt(gpioNumberStr);


            String setValueStr = editSetValue.getText().toString();
            if (TextUtils.isEmpty(setValueStr)) {
                showMessage("设置GPIO的输出电平不能为空");
                return;
            }
            if (setValueStr.equals("0") || setValueStr.equals("1")) {
                Log.i(TAG, "in mGpioSetValue value=" + setValueStr);
                int value = Integer.parseInt(setValueStr);
                boolean isSuccess = mNorcoAPI.hwctrl_gpioWrite(gpioNumber, value);
                Log.i(TAG, "in mGpioSetValue isSuccess=" + isSuccess);
            } else {
                showMessage("value需输入0/1");
            }
        }
    };

    private View.OnClickListener mGpioGetValue = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            String gpioNumberStr = editGpioNumber.getText().toString();
            if (TextUtils.isEmpty(gpioNumberStr)) {
                showMessage("请输入GPIO索引号");
                return;
            }
            if (!TextUtils.isDigitsOnly(gpioNumberStr)) {
                showMessage("请输入整数GPIO索引号");
                return;
            }
            int gpioNumber = Integer.parseInt(gpioNumberStr);


            int value = mNorcoAPI.hwctrl_gpioRead(gpioNumber);
            Log.i(TAG, "in mGpioGetValue value=" + value);
            editGetValue.setText(value + "");
        }
    };

    private View.OnClickListener mGpioSetEdge = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            String gpioNumberStr = editGpioNumber.getText().toString();
            if (TextUtils.isEmpty(gpioNumberStr)) {
                showMessage("请输入GPIO索引号");
                return;
            }
            if (!TextUtils.isDigitsOnly(gpioNumberStr)) {
                showMessage("请输入整数GPIO索引号");
                return;
            }
            int gpioNumber = Integer.parseInt(gpioNumberStr);

            String edge = (String)spinnerEdge.getSelectedItem();
            Log.i(Util.TAG, "in mGpioSetEdge edge=" + edge);
            boolean isSuccess = mNorcoAPI.hwctrl_gpioSetEdge(gpioNumber, edge);
            Log.i(Util.TAG, "in mGpioSetEdge isSuccess=" + isSuccess);
        }
    };

    private View.OnClickListener mGpioGetEdge = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            String gpioNumberStr = editGpioNumber.getText().toString();
            if (TextUtils.isEmpty(gpioNumberStr)) {
                showMessage("请输入GPIO索引号");
                return;
            }
            if (!TextUtils.isDigitsOnly(gpioNumberStr)) {
                showMessage("请输入整数GPIO索引号");
                return;
            }
            int gpioNumber = Integer.parseInt(gpioNumberStr);

            Log.i(Util.TAG, "in mGpioGetEdge edge");
            String edge = mNorcoAPI.hwctrl_gpioGetEdge(gpioNumber);
            Log.i(Util.TAG, "in mGpioGetEdge edge=" + edge);
            editGetEdge.setText(edge);
        }
    };

    private NorcoNative.IGpioEventsCallback mGpioEventsCallback = new NorcoNative.IGpioEventsCallback() {
        @Override
        public void onEventsCallback(int gpioNumber, int value, String reason) {
            // 示例，主线程操作数据
            runOnUiThread(new Runnable() {
                public void run() {
                    Log.i(Util.TAG, "GPIO" + gpioNumber + " onEventsCallback value=" + value +  " reason=" + reason);
                }
            });
        }
    };

    private View.OnClickListener mGpioStartInterrupt = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            String gpioNumberStr = editGpioNumber.getText().toString();
            if (TextUtils.isEmpty(gpioNumberStr)) {
                showMessage("请输入GPIO索引号");
                return;
            }
            if (!TextUtils.isDigitsOnly(gpioNumberStr)) {
                showMessage("请输入整数GPIO索引号");
                return;
            }
            int gpioNumber = Integer.parseInt(gpioNumberStr);
            int timeout = 200; // 单位毫秒

            Log.i(Util.TAG, "in mGpioStartInterrupt");
            boolean isSuccess = mNorcoAPI.hwctrl_gpioStartInterrupt(gpioNumber, timeout, mGpioEventsCallback);
            Log.i(Util.TAG, "in mGpioStartInterrupt isSuccess=" + isSuccess);
        }
    };

    private View.OnClickListener mGpioStopInterrupt = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            String gpioNumberStr = editGpioNumber.getText().toString();
            if (TextUtils.isEmpty(gpioNumberStr)) {
                showMessage("请输入GPIO索引号");
                return;
            }
            if (!TextUtils.isDigitsOnly(gpioNumberStr)) {
                showMessage("请输入整数GPIO索引号");
                return;
            }
            int gpioNumber = Integer.parseInt(gpioNumberStr);

            mNorcoAPI.hwctrl_gpioStopInterrupt(gpioNumber);
        }
    };

    // 显示Toast消息
    private void showMessage(String sMsg) {
        Toast.makeText(this, sMsg, Toast.LENGTH_SHORT).show();
    }
}
