package com.norco.apicustomertest;

import android.os.Bundle;

import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.ToggleButton;

import com.norco.API.NorcoAPI;

import java.util.ArrayList;
import java.util.List;

public class I2CActivity extends AppCompatActivity {

    private Button ButtonClear;
    private CheckBox checkBoxAutoClear;
    private EditText editTextReceiveData, editTextLines;

    private Button buttonSendI2C;
    private Button buttonReceiveI2C;
    private EditText editTextI2C;
    private ToggleButton toggleButtonI2C;

    private Spinner SpinnerI2C;
    private int iReceiveLines = 0;
    private String nodeA;

    private int slave_addr = 0x18;
    private int reg_addr = 0x20;
    private NorcoAPI mNorcoAPI = APIApplication.getApplication().getNorcoAPI();

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_hw_i2c);

        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.setHomeButtonEnabled(true);
            actionBar.setDisplayHomeAsUpEnabled(true);
        }
        setTitle(R.string.hw_i2c_test);

        initView();
        displayUIData();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case android.R.id.home:
            this.finish();
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onDestroy() {
        mNorcoAPI.hwctrl_i2cClose();
        super.onDestroy();
    }

    private void initView() {
        editTextReceiveData = (EditText)findViewById(R.id.editTextRecDisp);
        editTextLines = (EditText)findViewById(R.id.editTextLines);
        editTextI2C = (EditText)findViewById(R.id.editTextI2C);

        checkBoxAutoClear = (CheckBox)findViewById(R.id.checkBoxAutoClear);

        ButtonClear = (Button)findViewById(R.id.ButtonClear);
        buttonSendI2C = (Button)findViewById(R.id.ButtonSendI2C);
        buttonReceiveI2C = (Button)findViewById(R.id.ButtonReceiveI2C);
        toggleButtonI2C = (ToggleButton)findViewById(R.id.toggleButtonI2C);

        SpinnerI2C = (Spinner)findViewById(R.id.SpinnerI2C);
        ButtonClear.setOnClickListener(new ButtonClickEvent());
        buttonSendI2C.setOnClickListener(new ButtonClickEvent());
        buttonReceiveI2C.setOnClickListener(new ButtonClickEvent());

        toggleButtonI2C.setOnCheckedChangeListener(new ToggleButtonCheckedChangeEvent());

        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this, R.array.baudrates_value, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

        List<String> allDevices = new ArrayList<>();
        allDevices.add("/dev/i2c-0");
        allDevices.add("/dev/i2c-1");
        allDevices.add("/dev/i2c-2");
        allDevices.add("/dev/i2c-3");
        allDevices.add("/dev/i2c-4");
        ArrayAdapter<String> uartDevices = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, allDevices);
        uartDevices.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        SpinnerI2C.setAdapter(uartDevices);
        if (allDevices.size() > 0) {
            SpinnerI2C.setSelection(0);
            nodeA = SpinnerI2C.getSelectedItem().toString();
        }

        SpinnerI2C.setOnItemSelectedListener(new ItemSelectedEvent());
    }

    // i2c变化时，关闭打开的spi节点
    class ItemSelectedEvent implements Spinner.OnItemSelectedListener {
        public void onItemSelected(AdapterView<?> arg0, View arg1, int arg2, long arg3) {
            String temp = SpinnerI2C.getSelectedItem().toString();
            if (!nodeA.equals(temp)) {
                mNorcoAPI.hwctrl_i2cClose();
            }
            nodeA = temp;
            toggleButtonI2C.setChecked(false);
        }

        public void onNothingSelected(AdapterView<?> arg0) {

        }
    }

    // 清除按钮、发送、接收按钮
    class ButtonClickEvent implements View.OnClickListener {
        public void onClick(View v) {
            if (v == ButtonClear) {
                editTextReceiveData.setText("");
            } else if (v == buttonSendI2C) {
                sendData(editTextI2C.getText().toString());
            } else if (v == buttonReceiveI2C) {
                byte[] data = mNorcoAPI.hwctrl_i2cRead(slave_addr, reg_addr, 4);
                if (data != null) {
                    displayReceiveData(data);
                }
            }
        }
    }

    private class ToggleButtonCheckedChangeEvent implements ToggleButton.OnCheckedChangeListener {
        public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
            if (buttonView == toggleButtonI2C) {
                String node = SpinnerI2C.getSelectedItem().toString();
                if (isChecked) {
                    mNorcoAPI.hwctrl_i2cOpen(node);
                } else {
                    mNorcoAPI.hwctrl_i2cClose();
                }
            }
        }
    }

    // 界面数据
    private void displayUIData() {
        editTextI2C.setText("DATA");
    }

    // 显示接收数据
    private void displayReceiveData(byte[] data) {
        StringBuilder sMsg = new StringBuilder();
        sMsg.append(" [");
        sMsg.append(nodeA);
        sMsg.append("]");
        sMsg.append(bytesToHex(data));
        //sMsg.append(UartUtil.byteArrToHex(data));
        sMsg.append("\r\n");
        editTextReceiveData.append(sMsg);
        iReceiveLines++;
        editTextLines.setText(String.valueOf(iReceiveLines));
        if ((iReceiveLines > 500) && (checkBoxAutoClear.isChecked())) {
            editTextReceiveData.setText("");
            editTextLines.setText("0");
            iReceiveLines = 0;
        }
    }

    // 发送数据
    private void sendData(String data) {
        byte[] sendByte = data.getBytes();
        //byte[] sendByte = UartUtil.hexToByteArr(data);
        mNorcoAPI.hwctrl_i2cWrite(slave_addr, reg_addr, sendByte);
    }
    public static String bytesToHex(byte[] bytes) {
        StringBuilder buf = new StringBuilder(bytes.length * 2);
        for(byte b : bytes) { // 使用String的format方法进行转换
            buf.append(String.format("%02x", new Integer(b & 0xff)));
        }
        return buf.toString();
    }
}