package com.norco.apicustomertest;

import android.os.Bundle;
import android.os.Environment;

import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;
import android.text.TextUtils;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.norco.API.NorcoAPI;
import com.norco.API.callback.LogMakeTarCallback;
import com.norco.API.log.LogCommand;

public class LogControlActivity extends AppCompatActivity {

    private Button button2;
    private Button button3;
    private TextView mTextView;
    private EditText editTarPath;
    private EditText editUploadServer;

    private static final String TAG = "norcoAPI";
    private NorcoAPI mNorcoAPI = APIApplication.getApplication().getNorcoAPI();

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_log_control);

        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.setHomeButtonEnabled(true);
            actionBar.setDisplayHomeAsUpEnabled(true);
        }

        initView();
        setTitle(R.string.button_logcontrol);
        mNorcoAPI.norcoAPIRegister();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case android.R.id.home:
            this.finish(); // back button
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        mNorcoAPI.norcoAPIUnregister();
    }

    private void initView() {
        button2 = (Button)findViewById(R.id.button2);
        button2.setOnClickListener(getlogMakeTarPath);

        button3 = (Button)findViewById(R.id.button3);
        button3.setOnClickListener(logUploadTest);

        mTextView = (TextView)findViewById(R.id.log_save_path);
        String saveDir = Environment.getExternalStorageDirectory().getAbsolutePath() + "/Logs/";
        mTextView.setText("日志存储在" + saveDir);

        editTarPath = (EditText)findViewById(R.id.edit_tar_path);
        editUploadServer = (EditText)findViewById(R.id.edit_upload_server);
    }

    private View.OnClickListener getlogMakeTarPath = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            String cmd = LogCommand.SET_MAKE_NEW_LOG_TAR;
            mNorcoAPI.logctrl_setControlCmd(cmd, new LogMakeTarCallback() {
                @Override
                public void onResult(String savePath) {
                    Log.i(TAG, "logctrl_setControlCmd tarPath=" + savePath);
                    editTarPath.setText(savePath);
                }
            });
        }
    };

    private View.OnClickListener logUploadTest = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            String uploadServer = editUploadServer.getText().toString();
            if (TextUtils.isEmpty(uploadServer)) {
                showMessage("上传日志服务器地址不能为空");
                return;
            }
            String cmd = LogCommand.SET_LOG_UPLOAD_START + uploadServer;
            mNorcoAPI.logctrl_setControlCmd(cmd, null);
        }
    };

    // 显示Toast消息
    private void showMessage(String sMsg) {
        Toast.makeText(this, sMsg, Toast.LENGTH_SHORT).show();
    }
}