package com.norco.apicustomertest;

import android.Manifest;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;

import androidx.appcompat.app.AppCompatActivity;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.Toast;

import com.norco.API.NorcoAPI;

import androidx.annotation.NonNull;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

public class MainActivity extends AppCompatActivity {

    private Button button1;
    private Button button2;
    private Button button3;
    private Button button_networkcontrol;
    private Button button_hwcontrol;
    private Button button_logcontrol;

    private static final String TAG = "norcoAPI";
    private NorcoAPI mNorcoAPI = APIApplication.getApplication().getNorcoAPI();
    String[] mPermissions = new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.READ_PHONE_STATE};

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        initView();
        requestPermission();
        Log.i(TAG, "API Version=" + mNorcoAPI.norcoAPIVersion());
    }

    private void initView() {
        button1 = (Button)findViewById(R.id.button1);
        button1.setOnClickListener(HWInformationTest);

        button2 = (Button)findViewById(R.id.button2);
        button2.setOnClickListener(SystemControlTest);

        button3 = (Button)findViewById(R.id.button3);
        button3.setOnClickListener(DisplayControlTest);

        button_networkcontrol = (Button)findViewById(R.id.button_networkcontrol);
        button_networkcontrol.setOnClickListener(netWorkControlTest);

        button_hwcontrol = (Button)findViewById(R.id.button_hwcontrol);
        button_hwcontrol.setOnClickListener(HWControlTest);

        button_logcontrol = (Button)findViewById(R.id.button_logcontrol);
        button_logcontrol.setOnClickListener(logControlTest);
    }

    public void requestPermission() {
        if (Build.VERSION.SDK_INT >= 23) {
            boolean needapply = false;
            for (int id = 0; id < mPermissions.length; id++) {
                Log.i(TAG, "MainActivity checkSelfPermission id=" + id);
                int chechpermission = ContextCompat.checkSelfPermission(getApplicationContext(), mPermissions[id]);
                if (chechpermission != PackageManager.PERMISSION_GRANTED) {
                    Log.e(TAG, mPermissions[id] + " PERMISSION_DENIED");
                    needapply = true;
                } else {
                    Log.i(TAG, mPermissions[id] + " PERMISSION_GRANTED");
                }
            }
            if (needapply) {
                ActivityCompat.requestPermissions(MainActivity.this, mPermissions, 1);
            }
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        for (int i = 0; i < grantResults.length; i++) {
            if (grantResults[i] == PackageManager.PERMISSION_GRANTED) {
                Log.i(TAG, permissions[i] + "已授权");
                Toast.makeText(MainActivity.this, permissions[i] + " 已授权", Toast.LENGTH_SHORT).show();
            } else {
                Log.e(TAG, permissions[i] + "拒绝授权");
                Toast.makeText(MainActivity.this, permissions[i] + " 拒绝授权", Toast.LENGTH_SHORT).show();
            }
        }
    }

    private View.OnClickListener HWInformationTest = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            Log.i(TAG, "in button1 HWInformationTest");
            Intent intent = new Intent(MainActivity.this, HWInformationActivity.class);
            startActivity(intent);
        }
    };

    private View.OnClickListener SystemControlTest = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            Log.i(TAG, "in button2 SystemControlTest");
            Intent intent = new Intent(MainActivity.this, SystemControlActivity.class);
            startActivity(intent);
        }
    };

    private View.OnClickListener DisplayControlTest = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            Log.i(TAG, "in button2 DisplayControlTest");
            Intent intent = new Intent(MainActivity.this, DisplayControlActivity.class);
            startActivity(intent);
		}
	};

    private View.OnClickListener netWorkControlTest = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            Log.i(TAG, "in button_networkcontrol netWorkControlTest");
            Intent intent = new Intent(MainActivity.this, NetworkControlActivity.class);
            startActivity(intent);
        }
    };

    private View.OnClickListener HWControlTest = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            Log.i(TAG, "in button_hwcontrol HWControlTest");
            Intent intent = new Intent(MainActivity.this, HWControlActivity.class);
            startActivity(intent);
        }
    };

    private View.OnClickListener logControlTest = new View.OnClickListener() {
        @Override
        public void onClick(View arg0) {
            Log.i(TAG, "in logControlTest");
            Intent intent = new Intent(MainActivity.this, LogControlActivity.class);
            startActivity(intent);
        }
    };
}
