package com.norco.apicustomertest;

import android.os.Bundle;

import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;

import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;
import android.widget.ToggleButton;

import com.norco.API.NorcoAPI;

import java.util.ArrayList;
import java.util.List;

public class SPIActivity extends AppCompatActivity {

    private Button ButtonClear;
    private CheckBox checkBoxAutoClear;
    private EditText editTextReceiveData, editTextLines;

    private Button buttonSendSPI;
    private Button buttonReceiveSPI;
    private Button buttonTransferSPI;
    private EditText editTextSPI;
    private ToggleButton toggleButtonSPI;

    private Spinner SpinnerSPI;
    private int iReceiveLines = 0;
    private String nodeA;
    private static final String TAG = "norcoAPI";
    private NorcoAPI mNorcoAPI = APIApplication.getApplication().getNorcoAPI();

    private int mode = 3;
    private int bits = 8;
    private int speed = 1000000;
    private int delay = 10;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_hw_spi);

        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.setHomeButtonEnabled(true);
            actionBar.setDisplayHomeAsUpEnabled(true);
        }
        setTitle(R.string.hw_spi_test);

        initView();
        displayUIData();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case android.R.id.home:
            this.finish();
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onDestroy() {
        mNorcoAPI.hwctrl_spiClose();
        super.onDestroy();
    }

    private void initView() {
        editTextReceiveData = (EditText)findViewById(R.id.editTextRecDisp);
        editTextLines = (EditText)findViewById(R.id.editTextLines);
        editTextSPI = (EditText)findViewById(R.id.editTextSPI);

        checkBoxAutoClear = (CheckBox)findViewById(R.id.checkBoxAutoClear);

        ButtonClear = (Button)findViewById(R.id.ButtonClear);
        buttonSendSPI = (Button)findViewById(R.id.ButtonSendSPI);
        buttonReceiveSPI = (Button)findViewById(R.id.ButtonReceiveSPI);
        buttonTransferSPI = (Button)findViewById(R.id.ButtonTransferSPI);
        toggleButtonSPI = (ToggleButton)findViewById(R.id.toggleButtonSPI);

        SpinnerSPI = (Spinner)findViewById(R.id.SpinnerSPI);
        ButtonClear.setOnClickListener(new ButtonClickEvent());
        buttonSendSPI.setOnClickListener(new ButtonClickEvent());
        buttonReceiveSPI.setOnClickListener(new ButtonClickEvent());
        buttonTransferSPI.setOnClickListener(new ButtonClickEvent());
        toggleButtonSPI.setOnCheckedChangeListener(new ToggleButtonCheckedChangeEvent());

        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this, R.array.baudrates_value, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

        List<String> allDevices = new ArrayList<>();
        allDevices.add("/dev/spidev0.0");
        allDevices.add("/dev/spidev1.0");
        allDevices.add("/dev/spidev2.0");
        ArrayAdapter<String> uartDevices = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, allDevices);
        uartDevices.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        SpinnerSPI.setAdapter(uartDevices);

        if (allDevices.size() > 0) {
            SpinnerSPI.setSelection(0);
            nodeA = SpinnerSPI.getSelectedItem().toString();
        }

        SpinnerSPI.setOnItemSelectedListener(new ItemSelectedEvent());
    }

    // spi变化时，关闭打开的spi节点
    class ItemSelectedEvent implements Spinner.OnItemSelectedListener {
        public void onItemSelected(AdapterView<?> arg0, View arg1, int arg2, long arg3) {
            String temp = SpinnerSPI.getSelectedItem().toString();
            if (!nodeA.equals(temp)) {
                mNorcoAPI.hwctrl_spiClose();
            }
            nodeA = temp;
            toggleButtonSPI.setChecked(false);
        }

        public void onNothingSelected(AdapterView<?> arg0) {

        }
    }

    // 发送、接收按钮
    class ButtonClickEvent implements View.OnClickListener {
        public void onClick(View v) {
            if (v == ButtonClear) {
                editTextReceiveData.setText("");
            } else if (v == buttonSendSPI) {
                sendData(editTextSPI.getText().toString());
            } else if (v == buttonReceiveSPI) {
                byte[] data = mNorcoAPI.hwctrl_spiRead(4);
                if (data != null) {
                    displayReceiveData(data);
                }
            } else if (v == buttonTransferSPI) {
                transferData(editTextSPI.getText().toString());
//                byte[] writeBuf = {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f};
//                byte[] data = mNorcoAPI.hwctrl_spiTransfer(writeBuf, writeBuf.length, bits, speed, delay);
//                if (data != null) {
//                    displayReceiveData(data);
//                }
            }
        }
    }

    private class ToggleButtonCheckedChangeEvent implements ToggleButton.OnCheckedChangeListener {
        public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
            if (buttonView == toggleButtonSPI) {
                String node = SpinnerSPI.getSelectedItem().toString();
                if (isChecked) {
                    if (!mNorcoAPI.hwctrl_spiOpen(node, mode, bits, speed)) {
                        showMessage("打开失败，请检查权限");
                        buttonView.setChecked(false);
                    }
                } else {
                    mNorcoAPI.hwctrl_spiClose();
                }
            }
        }
    }

    // 显示Toast消息
    private void showMessage(String sMsg) {
        Toast.makeText(this, sMsg, Toast.LENGTH_SHORT).show();
    }

    // 界面数据
    private void displayUIData() {
        editTextSPI.setText("DATA");
    }

    // 显示接收数据
    private void displayReceiveData(byte[] data) {
        StringBuilder sMsg = new StringBuilder();
        sMsg.append(" [");
        sMsg.append(nodeA);
        sMsg.append("]");
        sMsg.append(bytesToHex(data));
        //sMsg.append(UartUtil.byteArrToHex(data));
        sMsg.append("\r\n");
        editTextReceiveData.append(sMsg);
        iReceiveLines++;
        editTextLines.setText(String.valueOf(iReceiveLines));
        if ((iReceiveLines > 500) && (checkBoxAutoClear.isChecked())) {
            editTextReceiveData.setText("");
            editTextLines.setText("0");
            iReceiveLines = 0;
        }
    }

    // 发送数据
    private void sendData(String data) {
        byte[] sendByte = data.getBytes();
        //byte[] sendByte = UartUtil.hexToByteArr(data);
        mNorcoAPI.hwctrl_spiWrite(sendByte);
    }

    private void transferData(String data) {
        byte[] sendByte = hexToBytes(data);
        byte[] rx_buf = mNorcoAPI.hwctrl_spiTransfer(sendByte, sendByte.length, bits, speed, delay);
        if (rx_buf != null) {
            displayReceiveData(rx_buf);
        }
    }

    public static String bytesToHex(byte[] bytes) {
        StringBuilder buf = new StringBuilder(bytes.length * 2);
        for(byte b : bytes) { // 使用String的format方法进行转换
            buf.append(String.format("%02x", new Integer(b & 0xff)));
        }
        return buf.toString();
    }

    public static byte[] hexToBytes(String str) {
        if(str == null || str.trim().equals("")) {
            return new byte[0];
        }

        byte[] bytes = new byte[str.length() / 2];
        for(int i = 0; i < str.length() / 2; i++) {
            String subStr = str.substring(i * 2, i * 2 + 2);
            bytes[i] = (byte) Integer.parseInt(subStr, 16);
        }

        return bytes;
    }
}