package com.norco.apicustomertest;

import android.content.res.Configuration;
import android.os.Bundle;

import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;
import android.text.InputType;
import android.text.method.KeyListener;
import android.text.method.NumberKeyListener;
import android.text.method.TextKeyListener;
import android.text.method.TextKeyListener.Capitalize;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.Spinner;
import android.widget.Toast;
import android.widget.ToggleButton;

import com.norco.API.NorcoAPI;
import com.norco.API.callback.UartReceiveDataCallback;
import com.norco.API.peripheral.uart.ReceiveDataBean;
import com.norco.API.util.ByteUtil;

import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

public class UartActivity extends AppCompatActivity {

    private Button ButtonClear;
    private CheckBox checkBoxAutoClear;
    private RadioButton radioButtonTxt, radioButtonHex;
    private EditText editTextReceiveData, editTextLines;

    private Button buttonSendCOMA, buttonSendCOMB, buttonSendCOMC, buttonSendCOMD;
    private EditText editTextCOMA, editTextCOMB, editTextCOMC, editTextCOMD;
    private EditText editTextTimeCOMA, editTextTimeCOMB, editTextTimeCOMC, editTextTimeCOMD;
    private CheckBox checkBoxAutoCOMA, checkBoxAutoCOMB, checkBoxAutoCOMC, checkBoxAutoCOMD;
    private ToggleButton toggleButtonCOMA, toggleButtonCOMB, toggleButtonCOMC, toggleButtonCOMD;

    private Spinner SpinnerCOMA, SpinnerCOMB, SpinnerCOMC, SpinnerCOMD;
    private Spinner SpinnerBaudRateCOMA, SpinnerBaudRateCOMB, SpinnerBaudRateCOMC, SpinnerBaudRateCOMD;

    private int iReceiveLines = 0;
    private DisplayReceiveDataThread mDisplayThread;
    private String nodeA, nodeB, nodeC, nodeD;
    private AutoSendThread uartAThread, uartBThread, uartCThread, uartDThread;

    private NorcoAPI mNorcoAPI = APIApplication.getApplication().getNorcoAPI();

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_uart);

        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.setHomeButtonEnabled(true);
            actionBar.setDisplayHomeAsUpEnabled(true);
        }
        setTitle(R.string.hw_uart_test);

        mDisplayThread = new DisplayReceiveDataThread();
        mDisplayThread.start();

        initView();
        displayUIData();
        getUartAvailableDevices();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case android.R.id.home:
            this.finish();
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onDestroy() {
        mNorcoAPI.hwctrl_uartReceiveStop(nodeA);
        mNorcoAPI.hwctrl_uartReceiveStop(nodeB);
        mNorcoAPI.hwctrl_uartReceiveStop(nodeC);
        mNorcoAPI.hwctrl_uartReceiveStop(nodeD);
        mNorcoAPI.hwctrl_uartClose(nodeA);
        mNorcoAPI.hwctrl_uartClose(nodeB);
        mNorcoAPI.hwctrl_uartClose(nodeC);
        mNorcoAPI.hwctrl_uartClose(nodeD);
        if (uartAThread != null) {
            uartAThread.setResume();
            uartAThread.setStoped();
            uartAThread = null;
        }
        if (uartBThread != null) {
            uartBThread.setResume();
            uartBThread.setStoped();
            uartBThread = null;
        }
        if (uartCThread != null) {
            uartCThread.setResume();
            uartCThread.setStoped();
            uartCThread = null;
        }
        if (uartDThread != null) {
            uartDThread.setResume();
            uartDThread.setStoped();
            uartDThread = null;
        }
        mDisplayThread.setStop(true);
        super.onDestroy();
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        mNorcoAPI.hwctrl_uartReceiveStop(nodeA);
        mNorcoAPI.hwctrl_uartReceiveStop(nodeB);
        mNorcoAPI.hwctrl_uartReceiveStop(nodeC);
        mNorcoAPI.hwctrl_uartReceiveStop(nodeD);
        mNorcoAPI.hwctrl_uartClose(nodeA);
        mNorcoAPI.hwctrl_uartClose(nodeB);
        mNorcoAPI.hwctrl_uartClose(nodeC);
        mNorcoAPI.hwctrl_uartClose(nodeD);
        setContentView(R.layout.activity_uart);
        initView();
        displayUIData();
    }

    private void initView() {
        editTextReceiveData = (EditText)findViewById(R.id.editTextRecDisp);
        editTextLines = (EditText)findViewById(R.id.editTextLines);
        editTextCOMA = (EditText)findViewById(R.id.editTextCOMA);
        editTextCOMB = (EditText)findViewById(R.id.editTextCOMB);
        editTextCOMC = (EditText)findViewById(R.id.editTextCOMC);
        editTextCOMD = (EditText)findViewById(R.id.editTextCOMD);
        editTextTimeCOMA = (EditText)findViewById(R.id.editTextTimeCOMA);
        editTextTimeCOMB = (EditText)findViewById(R.id.editTextTimeCOMB);
        editTextTimeCOMC = (EditText)findViewById(R.id.editTextTimeCOMC);
        editTextTimeCOMD = (EditText)findViewById(R.id.editTextTimeCOMD);

        checkBoxAutoClear = (CheckBox)findViewById(R.id.checkBoxAutoClear);
        checkBoxAutoCOMA = (CheckBox)findViewById(R.id.checkBoxAutoCOMA);
        checkBoxAutoCOMB = (CheckBox)findViewById(R.id.checkBoxAutoCOMB);
        checkBoxAutoCOMC = (CheckBox)findViewById(R.id.checkBoxAutoCOMC);
        checkBoxAutoCOMD = (CheckBox)findViewById(R.id.checkBoxAutoCOMD);
        ButtonClear = (Button)findViewById(R.id.ButtonClear);
        buttonSendCOMA = (Button)findViewById(R.id.ButtonSendCOMA);
        buttonSendCOMB = (Button)findViewById(R.id.ButtonSendCOMB);
        buttonSendCOMC = (Button)findViewById(R.id.ButtonSendCOMC);
        buttonSendCOMD = (Button)findViewById(R.id.ButtonSendCOMD);
        toggleButtonCOMA = (ToggleButton)findViewById(R.id.toggleButtonCOMA);
        toggleButtonCOMB = (ToggleButton)findViewById(R.id.ToggleButtonCOMB);
        toggleButtonCOMC = (ToggleButton)findViewById(R.id.ToggleButtonCOMC);
        toggleButtonCOMD = (ToggleButton)findViewById(R.id.ToggleButtonCOMD);
        SpinnerCOMA = (Spinner)findViewById(R.id.SpinnerCOMA);
        SpinnerCOMB = (Spinner)findViewById(R.id.SpinnerCOMB);
        SpinnerCOMC = (Spinner)findViewById(R.id.SpinnerCOMC);
        SpinnerCOMD = (Spinner)findViewById(R.id.SpinnerCOMD);
        SpinnerBaudRateCOMA = (Spinner)findViewById(R.id.SpinnerBaudRateCOMA);
        SpinnerBaudRateCOMB = (Spinner)findViewById(R.id.SpinnerBaudRateCOMB);
        SpinnerBaudRateCOMC = (Spinner)findViewById(R.id.SpinnerBaudRateCOMC);
        SpinnerBaudRateCOMD = (Spinner)findViewById(R.id.SpinnerBaudRateCOMD);
        radioButtonTxt = (RadioButton)findViewById(R.id.radioButtonTxt);
        radioButtonHex = (RadioButton)findViewById(R.id.radioButtonHex);

        radioButtonTxt.setOnClickListener(new radioButtonClickEvent());
        radioButtonHex.setOnClickListener(new radioButtonClickEvent());
        ButtonClear.setOnClickListener(new ButtonClickEvent());
        buttonSendCOMA.setOnClickListener(new ButtonClickEvent());
        buttonSendCOMB.setOnClickListener(new ButtonClickEvent());
        buttonSendCOMC.setOnClickListener(new ButtonClickEvent());
        buttonSendCOMD.setOnClickListener(new ButtonClickEvent());
        toggleButtonCOMA.setOnCheckedChangeListener(new ToggleButtonCheckedChangeEvent());
        toggleButtonCOMB.setOnCheckedChangeListener(new ToggleButtonCheckedChangeEvent());
        toggleButtonCOMC.setOnCheckedChangeListener(new ToggleButtonCheckedChangeEvent());
        toggleButtonCOMD.setOnCheckedChangeListener(new ToggleButtonCheckedChangeEvent());
        checkBoxAutoCOMA.setOnCheckedChangeListener(new CheckBoxChangeEvent());
        checkBoxAutoCOMB.setOnCheckedChangeListener(new CheckBoxChangeEvent());
        checkBoxAutoCOMC.setOnCheckedChangeListener(new CheckBoxChangeEvent());
        checkBoxAutoCOMD.setOnCheckedChangeListener(new CheckBoxChangeEvent());

        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this, R.array.baudrates_value, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        SpinnerBaudRateCOMA.setAdapter(adapter);
        SpinnerBaudRateCOMB.setAdapter(adapter);
        SpinnerBaudRateCOMC.setAdapter(adapter);
        SpinnerBaudRateCOMD.setAdapter(adapter);
        SpinnerBaudRateCOMA.setSelection(4);
        SpinnerBaudRateCOMB.setSelection(4);
        SpinnerBaudRateCOMC.setSelection(4);
        SpinnerBaudRateCOMD.setSelection(4);
    }

    private void getUartAvailableDevices() {
        AsyncTaskWrapper<Void, Void, List<String>> task = new AsyncTaskWrapper<>();
        task.setOnTaskListener(mTaskListener);
        task.executeOnExecutor(AsyncTaskWrapper.THREAD_POOL_CACHED);
    }

    private AsyncTaskWrapper.OnLoadListener<Void, Void, List<String>> mTaskListener = new AsyncTaskWrapper.OnLoadListener<Void, Void, List<String>>() {

        @Override
        public void onStart(Object taskTag) {

        }

        @Override
        public void onCancel(Object taskTag) {

        }

        @Override
        public void onResult(Object taskTag, List<String> allDevices) {
            ArrayAdapter<String> uartDevices = new ArrayAdapter<String>(UartActivity.this, android.R.layout.simple_spinner_item, allDevices);
            uartDevices.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
            SpinnerCOMA.setAdapter(uartDevices);
            SpinnerCOMB.setAdapter(uartDevices);
            SpinnerCOMC.setAdapter(uartDevices);
            SpinnerCOMD.setAdapter(uartDevices);
            if (allDevices.size() > 0) {
                SpinnerCOMA.setSelection(0);
                nodeA = SpinnerCOMA.getSelectedItem().toString();
            }
            if (allDevices.size() > 1) {
                SpinnerCOMB.setSelection(1);
                nodeB = SpinnerCOMB.getSelectedItem().toString();
            }
            if (allDevices.size() > 2) {
                SpinnerCOMC.setSelection(2);
                nodeC = SpinnerCOMC.getSelectedItem().toString();
            }
            if (allDevices.size() > 3) {
                SpinnerCOMD.setSelection(3);
                nodeD = SpinnerCOMD.getSelectedItem().toString();
            }
            SpinnerCOMA.setOnItemSelectedListener(new ItemSelectedEvent());
            SpinnerCOMB.setOnItemSelectedListener(new ItemSelectedEvent());
            SpinnerCOMC.setOnItemSelectedListener(new ItemSelectedEvent());
            SpinnerCOMD.setOnItemSelectedListener(new ItemSelectedEvent());
            SpinnerBaudRateCOMA.setOnItemSelectedListener(new ItemSelectedEvent());
            SpinnerBaudRateCOMB.setOnItemSelectedListener(new ItemSelectedEvent());
            SpinnerBaudRateCOMC.setOnItemSelectedListener(new ItemSelectedEvent());
            SpinnerBaudRateCOMD.setOnItemSelectedListener(new ItemSelectedEvent());
        }

        @Override
        public List<String> onWorkerThread(Object taskTag, Void... params) {
            return mNorcoAPI.hwctrl_uartAvailableDevices();
        }
    };

    // 串口号或波特率变化时，关闭打开的串口，停止自动发送
    class ItemSelectedEvent implements Spinner.OnItemSelectedListener {
        public void onItemSelected(AdapterView<?> arg0, View arg1, int arg2, long arg3) {
            if ((arg0 == SpinnerCOMA) || (arg0 == SpinnerBaudRateCOMA)) {
                if (mNorcoAPI.hwctrl_uartOpened(nodeA)) {
                    mNorcoAPI.hwctrl_uartReceiveStop(nodeA);
                    mNorcoAPI.hwctrl_uartClose(nodeA);
                }
                nodeA = SpinnerCOMA.getSelectedItem().toString();
                checkBoxAutoCOMA.setChecked(false);
                toggleButtonCOMA.setChecked(false);
            } else if ((arg0 == SpinnerCOMB) || (arg0 == SpinnerBaudRateCOMB)) {
                if (mNorcoAPI.hwctrl_uartOpened(nodeB)) {
                    mNorcoAPI.hwctrl_uartReceiveStop(nodeB);
                    mNorcoAPI.hwctrl_uartClose(nodeB);
                }
                nodeB = SpinnerCOMB.getSelectedItem().toString();
                checkBoxAutoCOMB.setChecked(false);
                toggleButtonCOMB.setChecked(false);
            } else if ((arg0 == SpinnerCOMC) || (arg0 == SpinnerBaudRateCOMC)) {
                if (mNorcoAPI.hwctrl_uartOpened(nodeC)) {
                    mNorcoAPI.hwctrl_uartReceiveStop(nodeC);
                    mNorcoAPI.hwctrl_uartClose(nodeC);
                }
                nodeC = SpinnerCOMC.getSelectedItem().toString();
                checkBoxAutoCOMC.setChecked(false);
                toggleButtonCOMC.setChecked(false);
            } else if ((arg0 == SpinnerCOMD) || (arg0 == SpinnerBaudRateCOMD)) {
                if (mNorcoAPI.hwctrl_uartOpened(nodeD)) {
                    mNorcoAPI.hwctrl_uartReceiveStop(nodeD);
                    mNorcoAPI.hwctrl_uartClose(nodeD);
                }
                nodeD = SpinnerCOMD.getSelectedItem().toString();
                checkBoxAutoCOMD.setChecked(false);
                toggleButtonCOMD.setChecked(false);
            }
        }

        public void onNothingSelected(AdapterView<?> arg0) {

        }
    }

    // Txt Hex模式选择
    private class radioButtonClickEvent implements RadioButton.OnClickListener {
        public void onClick(View v) {
            if (v == radioButtonTxt) {
                KeyListener TxtkeyListener = new TextKeyListener(Capitalize.NONE, false);
                editTextCOMA.setKeyListener(TxtkeyListener);
                editTextCOMB.setKeyListener(TxtkeyListener);
                editTextCOMC.setKeyListener(TxtkeyListener);
                editTextCOMD.setKeyListener(TxtkeyListener);
                editTextCOMA.setText(R.string.uart_send_texta);
                editTextCOMB.setText(R.string.uart_send_textb);
                editTextCOMC.setText(R.string.uart_send_textc);
                editTextCOMD.setText(R.string.uart_send_textd);
            } else if (v == radioButtonHex) {
                KeyListener HexkeyListener = new NumberKeyListener() {
                    public int getInputType() {
                        return InputType.TYPE_CLASS_TEXT;
                    }

                    @Override
                    protected char[] getAcceptedChars() {
                        return new char[]{'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f', 'A', 'B', 'C', 'D', 'E', 'F'};
                    }
                };
                editTextCOMA.setKeyListener(HexkeyListener);
                editTextCOMB.setKeyListener(HexkeyListener);
                editTextCOMC.setKeyListener(HexkeyListener);
                editTextCOMD.setKeyListener(HexkeyListener);
                editTextCOMA.setText(R.string.uart_send_hexa);
                editTextCOMB.setText(R.string.uart_send_hexb);
                editTextCOMC.setText(R.string.uart_send_hexc);
                editTextCOMD.setText(R.string.uart_send_hexd);
            }
        }
    }

    // 自动发送
    private class CheckBoxChangeEvent implements CheckBox.OnCheckedChangeListener {
        public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
            if (buttonView == checkBoxAutoCOMA) {
                if (!toggleButtonCOMA.isChecked() && isChecked) {
                    buttonView.setChecked(false);
                    return;
                }
                if (uartAThread == null) {
                    uartAThread = new AutoSendThread();
                    uartAThread.setSuspend();
                    uartAThread.start();
                }
                if (radioButtonTxt.isChecked()) {
                    uartAThread.setTxtLoopData(editTextCOMA.getText().toString());
                } else if (radioButtonHex.isChecked()) {
                    uartAThread.setHexLoopData(editTextCOMA.getText().toString());
                }
                String time = editTextTimeCOMA.getText().toString();
                uartAThread.setAutoDelay(Integer.parseInt(time));
                if (isChecked) {
                    uartAThread.setUartNode(nodeA);
                    uartAThread.setResume();
                } else {
                    uartAThread.setSuspend();
                }
            } else if (buttonView == checkBoxAutoCOMB) {
                if (!toggleButtonCOMB.isChecked() && isChecked) {
                    buttonView.setChecked(false);
                    return;
                }
                if (uartBThread == null) {
                    uartBThread = new AutoSendThread();
                    uartBThread.setSuspend();
                    uartBThread.start();
                }
                if (radioButtonTxt.isChecked()) {
                    uartBThread.setTxtLoopData(editTextCOMB.getText().toString());
                } else if (radioButtonHex.isChecked()) {
                    uartBThread.setHexLoopData(editTextCOMB.getText().toString());
                }
                String time = editTextTimeCOMB.getText().toString();
                uartBThread.setAutoDelay(Integer.parseInt(time));
                if (isChecked) {
                    uartBThread.setUartNode(nodeB);
                    uartBThread.setResume();
                } else {
                    uartBThread.setSuspend();
                }
            } else if (buttonView == checkBoxAutoCOMC) {
                if (!toggleButtonCOMC.isChecked() && isChecked) {
                    buttonView.setChecked(false);
                    return;
                }
                if (uartCThread == null) {
                    uartCThread = new AutoSendThread();
                    uartCThread.setSuspend();
                    uartCThread.start();
                }
                if (radioButtonTxt.isChecked()) {
                    uartCThread.setTxtLoopData(editTextCOMC.getText().toString());
                } else if (radioButtonHex.isChecked()) {
                    uartCThread.setHexLoopData(editTextCOMC.getText().toString());
                }
                String time = editTextTimeCOMC.getText().toString();
                uartCThread.setAutoDelay(Integer.parseInt(time));
                if (isChecked) {
                    uartCThread.setUartNode(nodeC);
                    uartCThread.setResume();
                } else {
                    uartCThread.setSuspend();
                }
            } else if (buttonView == checkBoxAutoCOMD) {
                if (!toggleButtonCOMD.isChecked() && isChecked) {
                    buttonView.setChecked(false);
                    return;
                }
                if (uartDThread == null) {
                    uartDThread = new AutoSendThread();
                    uartDThread.setSuspend();
                    uartDThread.start();
                }
                if (radioButtonTxt.isChecked()) {
                    uartDThread.setTxtLoopData(editTextCOMD.getText().toString());
                } else if (radioButtonHex.isChecked()) {
                    uartDThread.setHexLoopData(editTextCOMD.getText().toString());
                }
                String time = editTextTimeCOMD.getText().toString();
                uartDThread.setAutoDelay(Integer.parseInt(time));
                if (isChecked) {
                    uartDThread.setUartNode(nodeD);
                    uartDThread.setResume();
                } else {
                    uartDThread.setSuspend();
                }
            }
        }
    }

    // 清除按钮、发送按钮
    class ButtonClickEvent implements View.OnClickListener {
        public void onClick(View v) {
            if (v == ButtonClear) {
                editTextReceiveData.setText("");
            } else if (v == buttonSendCOMA) {
                sendData(nodeA, editTextCOMA.getText().toString().trim());
            } else if (v == buttonSendCOMB) {
                sendData(nodeB, editTextCOMB.getText().toString().trim());
            } else if (v == buttonSendCOMC) {
                sendData(nodeC, editTextCOMC.getText().toString().trim());
            } else if (v == buttonSendCOMD) {
                sendData(nodeD, editTextCOMD.getText().toString().trim());
            }
        }
    }

    private class ToggleButtonCheckedChangeEvent implements ToggleButton.OnCheckedChangeListener {
        public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
            if (buttonView == toggleButtonCOMA) {
                if (isChecked) {
                    if (toggleButtonCOMB.isChecked() && SpinnerCOMA.getSelectedItemPosition() == SpinnerCOMB.getSelectedItemPosition()) {
                        showMessage("串口" + SpinnerCOMA.getSelectedItem().toString() + "已打开");
                        buttonView.setChecked(false);
                    } else if (toggleButtonCOMC.isChecked() && SpinnerCOMA.getSelectedItemPosition() == SpinnerCOMC.getSelectedItemPosition()) {
                        showMessage("串口" + SpinnerCOMA.getSelectedItem().toString() + "已打开");
                        buttonView.setChecked(false);
                    } else if (toggleButtonCOMD.isChecked() && SpinnerCOMA.getSelectedItemPosition() == SpinnerCOMD.getSelectedItemPosition()) {
                        showMessage("串口" + SpinnerCOMA.getSelectedItem().toString() + "已打开");
                        buttonView.setChecked(false);
                    } else {
                        String node = SpinnerCOMA.getSelectedItem().toString();
                        String baudrate = SpinnerBaudRateCOMA.getSelectedItem().toString();
                        mNorcoAPI.hwctrl_uartOpen(node, Integer.parseInt(baudrate), 0, 8, 1, 'N', 0, 1);
                        mNorcoAPI.hwctrl_uartReceive(node, mDataCallback);
                    }
                } else {
                    String node = SpinnerCOMA.getSelectedItem().toString();
                    if (mNorcoAPI.hwctrl_uartOpened(node)) {
                        mNorcoAPI.hwctrl_uartReceiveStop(node);
                        mNorcoAPI.hwctrl_uartClose(node);
                    }
                    checkBoxAutoCOMA.setChecked(false);
                }
            } else if (buttonView == toggleButtonCOMB) {
                if (isChecked) {
                    if (toggleButtonCOMA.isChecked() && SpinnerCOMB.getSelectedItemPosition() == SpinnerCOMA.getSelectedItemPosition()) {
                        showMessage("串口" + SpinnerCOMB.getSelectedItem().toString() + "已打开");
                        buttonView.setChecked(false);
                    } else if (toggleButtonCOMC.isChecked() && SpinnerCOMB.getSelectedItemPosition() == SpinnerCOMC.getSelectedItemPosition()) {
                        showMessage("串口" + SpinnerCOMB.getSelectedItem().toString() + "已打开");
                        buttonView.setChecked(false);
                    } else if (toggleButtonCOMD.isChecked() && SpinnerCOMB.getSelectedItemPosition() == SpinnerCOMD.getSelectedItemPosition()) {
                        showMessage("串口" + SpinnerCOMB.getSelectedItem().toString() + "已打开");
                        buttonView.setChecked(false);
                    } else {
                        String node = SpinnerCOMB.getSelectedItem().toString();
                        String baudrate = SpinnerBaudRateCOMB.getSelectedItem().toString();
                        mNorcoAPI.hwctrl_uartOpen(node, Integer.parseInt(baudrate), 0, 8, 1, 'N', 0, 1);
                        mNorcoAPI.hwctrl_uartReceive(node, mDataCallback);
                    }
                } else {
                    String node = SpinnerCOMB.getSelectedItem().toString();
                    if (mNorcoAPI.hwctrl_uartOpened(node)) {
                        mNorcoAPI.hwctrl_uartReceiveStop(node);
                        mNorcoAPI.hwctrl_uartClose(node);
                    }
                    checkBoxAutoCOMB.setChecked(false);
                }
            } else if (buttonView == toggleButtonCOMC) {
                if (isChecked) {
                    if (toggleButtonCOMA.isChecked() && SpinnerCOMC.getSelectedItemPosition() == SpinnerCOMA.getSelectedItemPosition()) {
                        showMessage("串口" + SpinnerCOMC.getSelectedItem().toString() + "已打开");
                        buttonView.setChecked(false);
                    } else if (toggleButtonCOMB.isChecked() && SpinnerCOMC.getSelectedItemPosition() == SpinnerCOMB.getSelectedItemPosition()) {
                        showMessage("串口" + SpinnerCOMC.getSelectedItem().toString() + "已打开");
                        buttonView.setChecked(false);
                    } else if (toggleButtonCOMD.isChecked() && SpinnerCOMC.getSelectedItemPosition() == SpinnerCOMD.getSelectedItemPosition()) {
                        showMessage("串口" + SpinnerCOMC.getSelectedItem().toString() + "已打开");
                        buttonView.setChecked(false);
                    } else {
                        String node = SpinnerCOMC.getSelectedItem().toString();
                        String baudrate = SpinnerBaudRateCOMC.getSelectedItem().toString();
                        mNorcoAPI.hwctrl_uartOpen(node, Integer.parseInt(baudrate), 0, 8, 1, 'N', 0, 1);
                        mNorcoAPI.hwctrl_uartReceive(node, mDataCallback);
                    }
                } else {
                    String node = SpinnerCOMC.getSelectedItem().toString();
                    if (mNorcoAPI.hwctrl_uartOpened(node)) {
                        mNorcoAPI.hwctrl_uartReceiveStop(node);
                        mNorcoAPI.hwctrl_uartClose(node);
                    }
                    checkBoxAutoCOMC.setChecked(false);
                }
            } else if (buttonView == toggleButtonCOMD) {
                if (isChecked) {
                    if (toggleButtonCOMA.isChecked() && SpinnerCOMD.getSelectedItemPosition() == SpinnerCOMA.getSelectedItemPosition()) {
                        showMessage("串口" + SpinnerCOMD.getSelectedItem().toString() + "已打开");
                        buttonView.setChecked(false);
                    } else if (toggleButtonCOMB.isChecked() && SpinnerCOMD.getSelectedItemPosition() == SpinnerCOMB.getSelectedItemPosition()) {
                        showMessage("串口" + SpinnerCOMD.getSelectedItem().toString() + "已打开");
                        buttonView.setChecked(false);
                    } else if (toggleButtonCOMC.isChecked() && SpinnerCOMD.getSelectedItemPosition() == SpinnerCOMC.getSelectedItemPosition()) {
                        showMessage("串口" + SpinnerCOMD.getSelectedItem().toString() + "已打开");
                        buttonView.setChecked(false);
                    } else {
                        String node = SpinnerCOMD.getSelectedItem().toString();
                        String baudrate = SpinnerBaudRateCOMD.getSelectedItem().toString();
                        mNorcoAPI.hwctrl_uartOpen(node, Integer.parseInt(baudrate), 0, 8, 1, 'N', 0, 1);
                        mNorcoAPI.hwctrl_uartReceive(node, mDataCallback);
                    }
                } else {
                    String node = SpinnerCOMD.getSelectedItem().toString();
                    if (mNorcoAPI.hwctrl_uartOpened(node)) {
                        mNorcoAPI.hwctrl_uartReceiveStop(node);
                        mNorcoAPI.hwctrl_uartClose(node);
                    }
                    checkBoxAutoCOMD.setChecked(false);
                }
            }
        }
    }

    // 界面数据
    private void displayUIData() {
        editTextCOMA.setText(R.string.uart_send_hexa);
        editTextCOMB.setText(R.string.uart_send_hexb);
        editTextCOMC.setText(R.string.uart_send_hexc);
        editTextCOMD.setText(R.string.uart_send_hexd);
        editTextTimeCOMA.setText(R.string.uart_send_delay);
        editTextTimeCOMB.setText(R.string.uart_send_delay);
        editTextTimeCOMC.setText(R.string.uart_send_delay);
        editTextTimeCOMD.setText(R.string.uart_send_delay);
    }

    // 显示接收数据
    private void displayReceiveData(ReceiveDataBean bean) {
        StringBuilder sMsg = new StringBuilder();
        sMsg.append(bean.mReceiveTime);
        sMsg.append(" [");
        sMsg.append(bean.mNode);
        sMsg.append("]");
        if (radioButtonTxt.isChecked()) {
            sMsg.append(" [Txt] ");
            sMsg.append(new String(bean.mReceiveData));
        } else if (radioButtonHex.isChecked()) {
            sMsg.append(" [Hex] ");
            sMsg.append(ByteUtil.byteArrToHex(bean.mReceiveData));
        }
        sMsg.append("\n");
        editTextReceiveData.append(sMsg);
        iReceiveLines++;
        editTextLines.setText(String.valueOf(iReceiveLines));
        if ((iReceiveLines > 30) && (checkBoxAutoClear.isChecked())) {
            editTextReceiveData.setText("");
            editTextLines.setText("0");
            iReceiveLines = 0;
        }
    }

    // 发送数据
    private void sendData(String node, String data) {
        byte[] sendByte = null;
        data = data.replaceAll(" ", "");
        if (radioButtonTxt.isChecked()) {
            sendByte = data.getBytes();
        } else if (radioButtonHex.isChecked()) {
            sendByte = ByteUtil.hexToByteArr(data);
        }
        if (mNorcoAPI.hwctrl_uartOpened(node)) {
            mNorcoAPI.hwctrl_uartSend(node, sendByte);
        }
    }

    // 显示Toast消息
    private void showMessage(String sMsg) {
        Toast.makeText(this, sMsg, Toast.LENGTH_SHORT).show();
    }

    private UartReceiveDataCallback mDataCallback = new UartReceiveDataCallback() {
        @Override
        public void onReceiveData(final ReceiveDataBean ComRecData) {
            // 示例，主线程操作串口数据
            runOnUiThread(new Runnable() {
                public void run() {
                    mDisplayThread.addQueue(ComRecData);
                }
            });
        }
    };

    // 刷新显示线程
    private class DisplayReceiveDataThread extends Thread {

        private boolean mStop;
        private Queue<ReceiveDataBean> mReceiveQueue = new LinkedList<>();

        public void setStop(boolean stop) {
            mStop = stop;
        }

        public synchronized void addQueue(ReceiveDataBean data) {
            mReceiveQueue.add(data);
        }

        @Override
        public void run() {
            super.run();
            while (!mStop) {
                final ReceiveDataBean data;
                if ((data = mReceiveQueue.poll()) != null) {
                    runOnUiThread(new Runnable() {
                        public void run() {
                            displayReceiveData(data);
                        }
                    });
                } else {
                    try {
                        Thread.sleep(200);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    // 自动发送线程
    private class AutoSendThread extends Thread {

        private boolean mStoped;
        private boolean waitFlag = true;

        private byte[] mSendData;
        private int mAutoDelay = 500;
        private String uartNode;

        @Override
        public void run() {
            super.run();
            while (!mStoped) {
                synchronized (this) {
                    while (waitFlag) {
                        try {
                            wait();
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                    }
                }
                mNorcoAPI.hwctrl_uartSend(uartNode, mSendData);
                try {
                    Thread.sleep(mAutoDelay);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }

        public void setUartNode(String node) {
            uartNode = node;
        }

        public void setAutoDelay(int autoDelay) {
            mAutoDelay = autoDelay;
        }

        public void setTxtLoopData(String text) {
            mSendData = text.getBytes();
        }

        public void setHexLoopData(String hex) {
            mSendData = ByteUtil.hexToByteArr(hex);
        }

        public void setStoped() {
            mStoped = true;
        }

        public void setSuspend() {
            waitFlag = true;
        }

        public synchronized void setResume() {
            waitFlag = false;
            notifyAll();
        }
    }
}