#ifndef _API_NORCO_H
#define _API_NORCO_H

#ifdef  API_ANDROID
#define TAG "norcoAPI"
#include <android/log.h>
#define LOGI(fmt, args...) __android_log_print(ANDROID_LOG_INFO,  TAG, fmt, ##args)
#define LOGD(fmt, args...) __android_log_print(ANDROID_LOG_DEBUG, TAG, fmt, ##args)
#define LOGE(fmt, args...) __android_log_print(ANDROID_LOG_ERROR, TAG, fmt, ##args)
#endif

#ifdef  API_LINUX
#include <stdio.h>
#include <stdlib.h>
#define LOGI(fmt, args...) printf(fmt, ##args)
#define LOGD(fmt, args...) printf(fmt, ##args)
#define LOGE(fmt, args...) printf(fmt, ##args)
#endif

/**
 * 定时开机接口
 */
// 设置开机时间,成功返回0
int rtc_alarm_set(int day, int hour, int min);

// 使能定时开机,1使能,0关闭,成功返回0
int rtc_alarm_enable(int enabled);

// 获取rtc定时时间(CST时间)
int rtc_alarm_get(char *alarm_time);

// 获取rtc当前时间(CST时间)
int rtc_time_get(char *date_time);

/**
 * gpio操作接口
 */
// 导出,成功返回0,失败返回-1
int gpio_export(int gpioNumber);

// 取消导出,成功返回0,失败返回-1
int gpio_unexport(int gpioNumber);

// 读取电平,成功返回高低电平值,失败返回-1
int gpio_read(int gpioNumber);

// 设置电平,参数value: 0表示输出低电平 1表示输出高电平,成功返回0,失败返回-1
int gpio_write(int gpioNumber, int value);

// 设置方向,参数direction: 0表示输入引脚 1表示输出引脚,成功返回0,失败返回-1
int gpio_direction(int gpioNumber, int direction);

// 设置gpio中断的触发方式
int gpio_set_edge(int gpioNumber, const char *edge);

// 获取gpio中断的触发方式
int gpio_get_edge(int gpioNumber, char *edge);

// poll()事件回调
// 参数: gpioNumber, 电平值, 事件原因
typedef void (*GPIO_POLL_CALLBACK)(int, int, const char *);

// 使用poll()函数监听io口的实时电平变化
int gpio_poll_start(int gpioNumber, int timeout, GPIO_POLL_CALLBACK callback);

// 停止监听io口的实时电平变化
int gpio_poll_stop(int gpioNumber);

/**
 * i2c操作接口
 */
void i2c_close(int fd);
// 返回fd
int i2c_open(const char *node);
int i2c_read(int fd, unsigned char addr, unsigned char reg, unsigned char *val, unsigned char len);
int i2c_write(int fd, unsigned char addr, unsigned char reg, unsigned char *val, unsigned char len);

/**
 * spi操作接口
 */
void spi_close(int fd);
// 返回fd
int spi_open(const char *node);
int spi_read(int fd, unsigned char *buf, int len);
int spi_write(int fd, const char *buf, int len);
int spi_transfer(int fd, const char *tx_buf, int len, char *rx_buf);

/**
 * can操作接口
 */
// 返回fd
int can_open(const char *node);
// 关闭fd
void can_close(int fd);
// 失败返回-1,成功返回0
int can_read(int fd, unsigned char *buffer, unsigned int *read_len, unsigned long *can_id);
// 失败返回-1,成功返回写入的字节数
int can_write(int fd, unsigned long canid, const char *buffer, int len);

/**
 * 串口操作接口
 */
// 关闭fd
void uart_close(int fd);
// 返回fd,高级模式
int uart_open_advance(const char *node, int speed, int flow_ctrl, int databits, int stopbits, int parity, int vtime, int vmin);
// 失败返回-1,成功返回写入的字节数参数len
int uart_send(int fd, const char *send_buf, int data_len);
// 接收数据,返回实际读到的字节数
int uart_recv(int fd, char *recv_buf, int data_len);

#endif