package com.demo.test;

import java.io.File;
import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import com.norco.API.NorcoAPI;
import com.norco.API.NorcoNative;
import com.norco.API.callback.UartReceiveDataCallback;
import com.norco.API.peripheral.uart.ReceiveDataBean;
import com.norco.API.util.ByteUtil;

import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

import java.util.Date;
import java.text.SimpleDateFormat;

public class APILinuxDemo {

    private static NorcoAPI mNorcoAPI;
    private static boolean isHex = true;

    private static int gpioNumber = 150;
    private static String node = "/dev/ttyS0";

    public static void main(String[] args) {
        System.out.println("APILinuxJavaDemo start...");
        // 创建接口类对象
        mNorcoAPI = NorcoAPI.norcoAPICreate();

        // 开始GPIO测试
        gpioTest();

        // GPIO中断测试
        gpioInterruptTest();

        // 开始串口测试
        uartTest();
    }

    // GPIO测试
    private static void gpioTest() {
        System.out.println("\ngpioTest start...");
        // 释放一次(使用后未释放的情况)
        mNorcoAPI.hwctrl_gpioUnExport(gpioNumber);

        // 导出GPIO
        boolean isSuccess = mNorcoAPI.hwctrl_gpioExport(gpioNumber);
        System.out.println("in hwctrl_gpioExport isSuccess=" + isSuccess);

        // 配置为输出引脚
        isSuccess = mNorcoAPI.hwctrl_gpioDirection(gpioNumber, "out");
        System.out.println("in hwctrl_gpioDirection isSuccess=" + isSuccess);

        // 读取引脚电平
        int value = mNorcoAPI.hwctrl_gpioRead(gpioNumber);
        System.out.println("in hwctrl_gpioRead x1 value=" + value);

        // 输出高电平
        isSuccess = mNorcoAPI.hwctrl_gpioWrite(gpioNumber, 1);
        System.out.println("in hwctrl_gpioWrite isSuccess=" + isSuccess);

        // 读取引脚电平
        value = mNorcoAPI.hwctrl_gpioRead(gpioNumber);
        System.out.println("in hwctrl_gpioRead x2 value=" + value);

        // 取消导出GPIO
        isSuccess = mNorcoAPI.hwctrl_gpioUnExport(gpioNumber);
        System.out.println("in hwctrl_gpioUnExport isSuccess=" + isSuccess);
    }

    // GPIO中断数据回调接口
    private static SimpleDateFormat df = new SimpleDateFormat("HH:mm:ss SSS");
    private static NorcoNative.IGpioEventsCallback mGpioEventsCallback = new NorcoNative.IGpioEventsCallback() {
        @Override
        public void onEventsCallback(int gpioNumber, int value, String reason) {
            String time = df.format(new Date());
            System.out.println(time + " GPIO" + gpioNumber + " value=" + value + " onEventsCallback reason=" + reason);
        }
    };

    // GPIO中断测试
    private static void gpioInterruptTest() {
        System.out.println("\ngpioInterruptTest start...");
        // 释放一次(使用后未释放的情况)
        mNorcoAPI.hwctrl_gpioUnExport(gpioNumber);

        // 导出GPIO
        boolean isSuccess = mNorcoAPI.hwctrl_gpioExport(gpioNumber);
        System.out.println("in hwctrl_gpioExport isSuccess=" + isSuccess);

        // 配置为输入引脚
        isSuccess = mNorcoAPI.hwctrl_gpioDirection(gpioNumber, "in");
        System.out.println("in hwctrl_gpioDirection isSuccess=" + isSuccess);

        // 设置中断触发方式 "none" "rising" "falling" "both"
        isSuccess = mNorcoAPI.hwctrl_gpioSetEdge(gpioNumber, "both");
        System.out.println("in hwctrl_gpioSetEdge isSuccess=" + isSuccess);

        // 获取设置的中断触发方式
        String edge = mNorcoAPI.hwctrl_gpioGetEdge(gpioNumber);
        System.out.println("in hwctrl_gpioGetEdge edge=" + edge);

        // 超时时间, 单位为ms, 根据实际情况调整
        int timeout = 200;
        // 开启中断
        isSuccess = mNorcoAPI.hwctrl_gpioStartInterrupt(gpioNumber, timeout, mGpioEventsCallback);
        System.out.println("in hwctrl_gpioStartInterrupt isSuccess=" + isSuccess);

        // 测试外设运行(30分钟)
        try {
            Thread.sleep(30 * 60 * 1000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        // 停止中断
        mNorcoAPI.hwctrl_gpioStopInterrupt(gpioNumber);
    }

    // 串口收发测试
    private static void uartTest() {
        System.out.println("\nuartTest start...");
        mNorcoAPI.hwctrl_uartOpen(node, 115200, 0, 8, 1, 'N', 0, 1);
        mNorcoAPI.hwctrl_uartReceive(node, mDataCallback);

        AutoSendThread sendThread = new AutoSendThread();
        sendThread.start();
    }

    // 串口数据回调接口
    private static UartReceiveDataCallback mDataCallback = new UartReceiveDataCallback() {
        @Override
        public void onReceiveData(final ReceiveDataBean bean) {
            displayReceiveData(bean);
        }
    };

    // 发送测试
    private static class AutoSendThread extends Thread {
        @Override
        public void run() {
            super.run();
            for (int i = 0; i < 5; i++) {
                // 16进制字符串
                System.out.println("sendData...");
                sendData(node, "AA010203040506FF");
                try {
                    Thread.sleep(500);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }

            // 关闭串口示例
            /**mNorcoAPI.hwctrl_uartReceiveStop(node);
            if (mNorcoAPI.hwctrl_uartOpened(node)) {
                mNorcoAPI.hwctrl_uartClose(node);
            }*/
        }
    }

    // 发送数据
    private static void sendData(String node, String data) {
        byte[] sendByte = null;
        if (isHex) {
            sendByte = ByteUtil.hexToByteArr(data);
        } else {
            sendByte = data.getBytes();
        }
        if (mNorcoAPI.hwctrl_uartOpened(node)) {
            mNorcoAPI.hwctrl_uartSend(node, sendByte);
        }
    }

    // 显示接收数据
    private static void displayReceiveData(ReceiveDataBean bean) {
        StringBuilder sMsg = new StringBuilder();
        sMsg.append("[");
        sMsg.append(bean.mNode);
        sMsg.append("]");
        if (isHex) {
            sMsg.append(" [Hex] ");
            sMsg.append(ByteUtil.byteArrToHex(bean.mReceiveData));
        } else {
            sMsg.append(" [Txt] ");
            sMsg.append(new String(bean.mReceiveData));
        }
        sMsg.append("\n");
        System.out.println(sMsg);
    }
}
